package net.eusashead.hateoas.argumentresolver;

/*
 * #[license]
 * spring-responseentitybuilder
 * %%
 * Copyright (C) 2013 Eusa's Head
 * %%
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * %[license]
 */

import java.util.HashSet;
import java.util.Set;

import javax.servlet.http.HttpServletRequest;

import net.eusashead.hateoas.response.DeleteResponseBuilder;
import net.eusashead.hateoas.response.GetResponseBuilder;
import net.eusashead.hateoas.response.HeadResponseBuilder;
import net.eusashead.hateoas.response.OptionsResponseBuilder;
import net.eusashead.hateoas.response.PostResponseBuilder;
import net.eusashead.hateoas.response.PutResponseBuilder;
import net.eusashead.hateoas.response.ResponseBuilder;
import net.eusashead.hateoas.response.impl.DeleteResponseBuilderImpl;
import net.eusashead.hateoas.response.impl.GetResponseBuilderImpl;
import net.eusashead.hateoas.response.impl.HeadResponseBuilderImpl;
import net.eusashead.hateoas.response.impl.OptionsResponseBuilderImpl;
import net.eusashead.hateoas.response.impl.PostResponseBuilderImpl;
import net.eusashead.hateoas.response.impl.PutResponseBuilderImpl;

import org.springframework.core.MethodParameter;
import org.springframework.web.bind.support.WebDataBinderFactory;
import org.springframework.web.context.request.NativeWebRequest;
import org.springframework.web.method.support.HandlerMethodArgumentResolver;
import org.springframework.web.method.support.ModelAndViewContainer;

public class ResponseBuilderMethodArgumentResolver implements HandlerMethodArgumentResolver {

	private final Set<Class<?>> supported = new HashSet<Class<?>>();
	
	public ResponseBuilderMethodArgumentResolver() {
		this.supported.add(GetResponseBuilder.class);
		this.supported.add(HeadResponseBuilder.class);
		this.supported.add(OptionsResponseBuilder.class);
		this.supported.add(PostResponseBuilder.class);
		this.supported.add(PutResponseBuilder.class);
		this.supported.add(DeleteResponseBuilder.class);
	}
	
	@Override
	public boolean supportsParameter(MethodParameter parameter) {
		return supported.contains(parameter.getParameterType());
	}

	@Override
	public Object resolveArgument(MethodParameter parameter,
			ModelAndViewContainer mavContainer, NativeWebRequest webRequest,
			WebDataBinderFactory binderFactory) throws Exception {
		HttpServletRequest request = webRequest.getNativeRequest(HttpServletRequest.class);
		ResponseBuilder<?> builder;
		Class<?> parameterType = parameter.getParameterType();
		if (GetResponseBuilder.class.equals(parameterType)) {
			builder = new GetResponseBuilderImpl<Object>(request);
		} else if (HeadResponseBuilder.class.equals(parameterType)) {
			builder = new HeadResponseBuilderImpl<Object>(request);
		} else if (OptionsResponseBuilder.class.equals(parameterType)) {
			builder = new OptionsResponseBuilderImpl<Object>(request);
		} else if (PostResponseBuilder.class.equals(parameterType)) {
			builder = new PostResponseBuilderImpl(request);
		} else if (PutResponseBuilder.class.equals(parameterType)) {
			builder = new PutResponseBuilderImpl<Object>(request);
		} else if (DeleteResponseBuilder.class.equals(parameterType)) {
			builder = new DeleteResponseBuilderImpl(request);
		} else {
			throw new IllegalArgumentException("Cannot resolve argument of type " + parameterType);
		}
		return builder;
	}

}
