package net.eusashead.hateoas.response;

/*
 * #[license]
 * spring-responseentitybuilder
 * %%
 * Copyright (C) 2013 Eusa's Head
 * %%
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * %[license]
 */

import java.util.Date;

import net.eusashead.hateoas.header.ETagHeaderStrategy;

import org.springframework.http.ResponseEntity;

/**
 * Create an appropriate 
 * response to a GET request.
 * 
 * The returned {@link ResponseEntity}
 * should have status code 200 and
 * should have a body of type T, headers
 * with ETags, Last-Modified, Expires and 
 * Cache-Control headers
 *
 * @author patrickvk
 *
 * @param <T>
 */
public interface GetResponseBuilder<T> extends ResponseBuilder<T> {
	
	/**
	 * Set the wrapped entity
	 * that this response represents
	 * @param entity
	 */
	GetResponseBuilder<T> entity(T entity);

	/**
	 * Create ETag header based
	 * on the best available
	 * {@link ETagStrategy}
	 * @return
	 */
	GetResponseBuilder<T> etag();
	
	/**
	 * Create ETag header based
	 * on the supplied
	 * {@link ETagStrategy}
	 * @return
	 */
	GetResponseBuilder<T> etag(ETagHeaderStrategy strategy);

	/**
	 * Create a weak ETag based
	 * on the supplied date
	 * @param date
	 * @return
	 */
	GetResponseBuilder<T> etag(Date date);

	/**
	 * Create a strong ETag based
	 * on the supplied version
	 * number
	 * @param version
	 * @return
	 */
	GetResponseBuilder<T> etag(Integer version);
	
	/**
	 * Set the Last-Modified header
	 * based on the supplied date
	 * @param date
	 * @return
	 */
	GetResponseBuilder<T> lastModified(Date date);

	
	/**
	 * Set the the Expires and
	 * Cache-Control headers
	 * based on the supplied 
	 * number of milliseconds
	 * @param millis
	 * @return
	 */
	GetResponseBuilder<T> expireIn(long millis);
	
}
