package net.eusashead.hateoas.response;

/*
 * #[license]
 * spring-responseentitybuilder
 * %%
 * Copyright (C) 2013 Eusa's Head
 * %%
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * %[license]
 */

/**
 * Create an appropriate response
 * to a POST request.
 * The returned entity
 * should have a status code of 
 * 201 (created), a Location header 
 * and a null body.
 * 
 * @author patrickvk
 *
 * @param <T>
 */
public interface PostResponseBuilder extends ResponseBuilder<Void> {
	
	/** 
	 * Set the location header
	 * for use when creating
	 * a new resource using the 
	 * URI template and parameters
	 * to replace the template variables
	 *
	 * @param A {@link String} uriTemplate a URI template of the form /path/to/{variable}/{otherVariable}
	 * @param objects objects that will be interpolated into the URI template
	 * @return PostResponseBuilder for fluency
	 */
	PostResponseBuilder location(String uriTemplate, Object... objects);

	/**
	 * Set the location header
	 * with a static URI string
	 * @param uri a static {@link String} representing the URI
	 * @return PostResponseBuilder for fluency
	 */
	PostResponseBuilder location(String uri);

}
