package net.eusashead.hateoas.response.impl;

/*
 * #[license]
 * spring-responseentitybuilder
 * %%
 * Copyright (C) 2013 Eusa's Head
 * %%
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * %[license]
 */

import java.util.Date;

import javax.servlet.http.HttpServletRequest;

import net.eusashead.hateoas.header.ETagHeaderStrategy;
import net.eusashead.hateoas.header.impl.CacheControlHeaderImpl;
import net.eusashead.hateoas.header.impl.ETagHeaderImpl;
import net.eusashead.hateoas.header.impl.ExpiresHeaderImpl;
import net.eusashead.hateoas.header.impl.HashingETagHeaderStrategy;
import net.eusashead.hateoas.header.impl.LastModifiedHeaderImpl;

public abstract class AbstractEntityResponseBuilder<T,V>
extends AbstractResponseBuilder<T>  {

	protected V entity;

	public AbstractEntityResponseBuilder(
			HttpServletRequest request) {
		super(request);
	}

	protected void setEntity(V entity) {
		this.entity = entity;
		assertEntity();
	}

	protected void assertEntity() {
		if (entity == null) {
			throw new IllegalArgumentException("Entity T cannot be null.");
		}
	}

	protected void setEtagHeader() {
		assertEntity();
		this.headers.setETag(new HashingETagHeaderStrategy().extract(entity).toString()); 
	}

	protected void setEtagHeader(ETagHeaderStrategy etagStrategy) {
		assertEntity();
		this.headers.setETag(etagStrategy.extract(entity).toString()); 
	}

	protected void setEtagHeader(Date date) {
		// Weak ETag
		this.headers.setETag(new ETagHeaderImpl(date).toString()); 
	}

	protected void setEtagHeader(Integer version) {
		// Strong ETag
		this.headers.setETag(new ETagHeaderImpl(version).toString());
	}

	protected void setLastModifiedHeader(Date date) {
		this.headers.setLastModified(new LastModifiedHeaderImpl(date).getValue());
	}

	protected void setExpiryHeaders(long millis) {
		Date now = new Date();
		this.headers.setDate(now.getTime());
		this.headers.setExpires(new ExpiresHeaderImpl(now, millis).getValue());
		this.headers.setCacheControl(new CacheControlHeaderImpl(millis).getValue());
	}

	/**
	 * Compare the supplied ETag
	 * with the ETag in the
	 * request
	 * @param eTag
	 * @return
	 */
	protected boolean compareEtagWithIfNoneMatch(String eTag) {
		return checkEtag(eTag, "If-None-Match");
	}

	/**
	 * Compare outgoing ETag
	 * to incoming "If-None-Match"
	 * header and return true
	 * if they match
	 * @param eTag
	 * @return
	 */
	private boolean checkEtag(String eTag, String header) {
		String providedTag = request.getHeader(header);
		if (providedTag == null || !eTag.equals(providedTag)) {
			return false;
		} else {
			return true;
		}
	}

}